import java.nio.charset.StandardCharsets
import java.util.Base64
import javax.ws.rs.core.MediaType
import com.sun.jersey.api.client.Client
import com.sun.jersey.api.client.ClientResponse
import com.sun.jersey.api.client.WebResource

object DecoderServiceDecodingExample {

  def main( args: Array[String] ): Unit = {

    // set proxy setting
    val PROXY_HOST: String = "rb-proxy-de.bosch.com"
    val PROXY_PORT: String = "8080"
    setProxySettings( PROXY_HOST, PROXY_PORT )

    // set user credentials
    val username: String = "your_username"
    val password: String = "your_password"
    val authorizationCredentials: String = generateAuthorizationToken(username, password)

    // prepare and send http post request
    val project: String = "demo"
    val `type`: String = "FIBEX"
    val decoderId: String = "your_decoder_ID"
    val hexInput: String = "your_HEX_input"

    val resourceUrl: String = "https://www.bosch-sfde.com/data-decoder-service/v1/" +
      project + "/decoders/" + decoderId + "/" + `type` + "/test"

    val webResource: WebResource = Client.create().resource( resourceUrl )
    val clientResponse: ClientResponse = webResource.accept( MediaType.WILDCARD_TYPE).`type`(MediaType.APPLICATION_JSON_TYPE )
      .header("Authorization", authorizationCredentials )
      .post( classOf[ClientResponse], "{\"testDataWithPdu\":[\"" + hexInput + "\"]}" )

    println( clientResponse.getStatus )
    println( clientResponse.getEntity( classOf[String] ) )

  }

  private def setProxySettings( host: String, port: String ): Unit = {
    System.setProperty( "http.proxyHost", host )
    System.setProperty( "http.proxyPort", port )
    System.setProperty( "https.proxyHost", host )
    System.setProperty( "https.proxyPort", port )
  }

  private def generateAuthorizationToken( username: String, password: String ): String =
    "Basic " + new String( Base64.getEncoder.encode( (username + ":" + password).getBytes ), StandardCharsets.UTF_8 )

}