import java.io.File
import java.nio.charset.StandardCharsets
import java.util.Base64
import javax.ws.rs.core.MediaType
import org.glassfish.jersey.media.multipart.FormDataMultiPart
import org.glassfish.jersey.media.multipart.file.FileDataBodyPart
import org.glassfish.jersey.media.multipart.internal.MultiPartWriter
import com.sun.jersey.api.client.Client
import com.sun.jersey.api.client.ClientResponse
import com.sun.jersey.api.client.WebResource
import com.sun.jersey.api.client.config.DefaultClientConfig

object DecoderServiceUploadExample {

  def main( args: Array[String] ): Unit = {
    // set proxy settings
    val PROXY_HOST: String = "rb-proxy-de.bosch.com"
    val PROXY_PORT: String = "8080"
    setProxySettings(PROXY_HOST, PROXY_PORT)

    // set user credentials
    val username: String = "your_username"
    val password: String = "your_password"
    val authorizationCredentials: String = generateAuthorizationToken( username, password )

    // prepare form data upload
    val project: String = "demo"
    val `type`: String = "FIBEX"
    val name: String = "my-new-decoder-file"
    val comment: String = "new new decoder file"
    val file: File = new File( "path_to_your_decoder_spec_file" )

    val resourceUrl: String = "https://www.bosch-sfde.com/data-decoder-service/v1/" + project + "/decoders"

    val defaultClientConfig: DefaultClientConfig = new DefaultClientConfig()
    defaultClientConfig.getClasses.add( classOf[MultiPartWriter] )
    val webResource: WebResource = Client.create( defaultClientConfig ).resource( resourceUrl )

    val formDataMultiPart: FormDataMultiPart = new FormDataMultiPart()
      .field( "type", `type` )
      .field( "project", project )
      .field( "name", name )
      .field( "comment", comment )
      .bodyPart( new FileDataBodyPart( "file", file ) )
      .asInstanceOf[FormDataMultiPart]

    // send http post request
    val clientResponse: ClientResponse = webResource
      .accept( MediaType.WILDCARD_TYPE )
      .`type`( MediaType.MULTIPART_FORM_DATA_TYPE )
      .header("Authorization", authorizationCredentials )
      .post( classOf[ClientResponse], formDataMultiPart )
    formDataMultiPart.close()

    println( clientResponse.getStatus )
    println( clientResponse.getEntity( classOf[String] ) )
  }

  private def setProxySettings( host: String, port: String ): Unit = {
    System.setProperty( "http.proxyHost", host )
    System.setProperty( "http.proxyPort", port )
    System.setProperty( "https.proxyHost", host )
    System.setProperty( "https.proxyPort", port )
  }

  private def generateAuthorizationToken( username: String, password: String ): String =
    "Basic " + new String( Base64.getEncoder.encode( (username + ":" + password).getBytes ), StandardCharsets.UTF_8 )

}