import java.io.InputStream
import java.io.InputStreamReader
import java.io.Reader
import java.nio.charset.StandardCharsets
import java.nio.file.Files
import java.nio.file.Path
import java.nio.file.Paths
import java.nio.file.StandardCopyOption
import com.google.gson.Gson
import com.google.gson.JsonObject
import com.sun.jersey.api.client.Client
import com.sun.jersey.api.client.ClientResponse
import com.sun.jersey.api.client.WebResource
import com.sun.jersey.core.util.Base64

object MongoDBQueryServiceAsyncExample {

  def main( args: Array[String] ): Unit = {

    val proxyHost: String = "rb-proxy-de.bosch.com"
    val proxyPort: String = "8080"

    val sfdeProject: String = "demo"
    val resourceUrl: String = "https://bosch-iot-insights.com/mongodb-query-service/v2/" + sfdeProject

    val username: String = "your_username"
    val password: String = "your_password"
    val authorizationCredentials: String = generateAuthorizationToken(username, password)

    val contentType: String = "application/json"
    val payload: String = new String( Files.readAllBytes( Paths.get( "src/resources/queryParametersAsync.json" ) ) )

    System.setProperty( "https.proxyHost", proxyHost )
    System.setProperty( "https.proxyPort", proxyPort )

    val response: ClientResponse = httpRequestPost( resourceUrl + "/submit-aggregation-query", authorizationCredentials, contentType, payload )

    println( response )

    if ( response.getStatus == 200 ) {
      var queryError: Boolean = false
      val responseContent: String = response.getEntity( classOf[String] )

      val json: JsonObject = new Gson().fromJson( responseContent, classOf[JsonObject] )
      val requestId: String = json.get( "queryId" ).getAsString
      var queryStatus: String = json.get( "status" ).getAsString

      println( "Status: " + queryStatus )

      while ( queryStatus.!=( "SUCCESSFUL" ) ) {
        val responseStream: InputStream = httpRequestGet( resourceUrl + "/queries" + "/" + requestId, authorizationCredentials,
          contentType )
        val reader: Reader = new InputStreamReader( responseStream )
        val jsonRes2: JsonObject = new Gson().fromJson( reader, classOf[JsonObject] )
        queryStatus = jsonRes2.get( "status" ).getAsString
        println( "Status: " + queryStatus )

        if ( queryStatus.==( "FAILED" ) || queryStatus.==( "INCORRECT" ) ) {
          queryError = true
        }
      }

      if ( !queryError ) {
        val responseStream: InputStream = httpRequestGet( resourceUrl + "/queries" + "/" + requestId + "/result", authorizationCredentials,
          contentType )
        println( "Status: LOADING DATA..." )
        val path: Path = Paths.get(System.getProperty( "user.home" ) + "/queryResponse.txt" )
        Files.copy( responseStream, path, StandardCopyOption.REPLACE_EXISTING )
        println( "Status: DONE" )
        println( "Response data was saved at " + path )
      }
    }
  }

  private def httpRequestPost( url: String, auth: String, cType: String, payload: String ): ClientResponse = {
    val service: WebResource = Client.create().resource( url )
    val response: ClientResponse =
      service.header( "Authorization", auth).header( "Content-Type", cType ).post( classOf[ClientResponse], payload )
    response
  }

  private def httpRequestGet( url: String, auth: String, cType: String ): InputStream = {
    val service: WebResource = Client.create().resource( url )
    val response: InputStream =
      service.header( "Authorization", auth ).header( "Content-Type", cType).get( classOf[InputStream] )
    response
  }

  private def generateAuthorizationToken( username: String, password: String ): String =
    "Basic " + new String(Base64.encode(username + ":" + password), StandardCharsets.UTF_8 )

}
