import java.nio.charset.StandardCharsets
import java.nio.file.Files
import java.nio.file.Paths
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import com.google.gson.JsonElement
import com.google.gson.JsonParser
import com.sun.jersey.api.client.Client
import com.sun.jersey.api.client.ClientResponse
import com.sun.jersey.api.client.WebResource
import com.sun.jersey.core.util.Base64

object MongoDBQueryServiceSyncExample {

  def main( args: Array[String] ): Unit = {

    val proxyHost: String = "rb-proxy-de.bosch.com"
    val proxyPort: String = "8080"

    val sfdeProject: String = "demo"
    val resourceUrl: String = "https://bosch-iot-insights.com/mongodb-query-service/v2/" + sfdeProject + "/execute-aggregation-query"

    val username: String = "your_username"
    val password: String = "your_password"
    val authorizationCredentials: String = generateAuthorizationToken( username, password )

    val contentType: String = "application/json"
    val payload: String = new String( Files.readAllBytes( Paths.get( "src/resources/queryParametersSync.json" )))

    System.setProperty( "https.proxyHost", proxyHost )
    System.setProperty( "https.proxyPort", proxyPort )

    val service: WebResource = Client.create().resource( resourceUrl )
    val response: ClientResponse = service.header( "Authorization", authorizationCredentials )
      .header( "Content-Type", contentType )
      .post( classOf[ClientResponse ], payload)

    println( response )

    if ( response.getStatus == 200 ) {
      println( parseJson( response.getEntity( classOf[String] ) ) )
    }
  }

  private def generateAuthorizationToken( username: String, password: String ): String =
    "Basic " + new String( Base64.encode( username + ":" + password ), StandardCharsets.UTF_8 )

  private def parseJson(plainString: String): String = {
    val gson: Gson = new GsonBuilder().setPrettyPrinting().create()
    val json: JsonElement = new JsonParser().parse( plainString )
    gson.toJson( json )
  }

}